@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2016 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,20 +20,20 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 import calendar
-import datetime
-from locale import getlocale, setlocale, LC_ALL
+import datetime as dt
+from locale import LC_ALL, LC_TIME, getlocale, setlocale
 
 from click import style
 
 from .terminal import colored
-
+from .utils import get_month_abbr_len
 
 setlocale(LC_ALL, '')
 
 
 def get_weekheader(firstweekday):
     try:
-        mylocale = '.'.join(getlocale())
+        mylocale = '.'.join(getlocale(LC_TIME))
     except TypeError:
         mylocale = 'C'
 
@@ -48,25 +48,44 @@ def getweeknumber(date):
     :return: weeknumber
     :rtype: int
     """
-    return datetime.date.isocalendar(date)[1]
+    return dt.date.isocalendar(date)[1]
 
 
-def get_event_color(event, default_color):
+def get_calendar_color(calendar, default_color, collection):
     """Because multi-line lambdas would be un-Pythonic
     """
-    if event.color == '':
+    if collection._calendars[calendar]['color'] == '':
         return default_color
-    return event.color
+    return collection._calendars[calendar]['color']
 
 
-def str_highlight_day(day, devents, hmethod, default_color, multiple, color, bold_for_light_color):
+def get_color_list(calendars, default_color, collection):
+    """Get the list of possible colors for the day, taking into account priority"""
+    dcolors = [
+        (
+            get_calendar_color(x, default_color, collection),
+            collection._calendars[x]["priority"],
+        )
+        for x in calendars
+    ]
+
+    dcolors.sort(key=lambda x: x[1], reverse=True)
+
+    maxPriority = dcolors[0][1]
+    dcolors = {x[0] for x in filter(lambda x: x[1] == maxPriority, dcolors)}
+
+    return list(dcolors)
+
+
+def str_highlight_day(day, calendars, hmethod, default_color, multiple, multiple_on_overflow,
+                      color, bold_for_light_color, collection):
     """returns a string with day highlighted according to configuration
     """
     dstr = str(day.day).rjust(2)
     if color == '':
-        dcolors = list(set(map(lambda x: get_event_color(x, default_color), devents)))
+        dcolors = get_color_list(calendars, default_color, collection)
         if len(dcolors) > 1:
-            if multiple == '':
+            if multiple == '' or (multiple_on_overflow and len(dcolors) == 2):
                 if hmethod == "foreground" or hmethod == "fg":
                     return colored(dstr[:1], fg=dcolors[0],
                                    bold_for_light_color=bold_for_light_color) + \
@@ -78,10 +97,7 @@ def str_highlight_day(day, devents, hmethod, default_color, multiple, color, bol
             else:
                 dcolor = multiple
         else:
-            if devents[0].color == '':
-                dcolor = default_color
-            else:
-                dcolor = devents[0].color
+            dcolor = dcolors[0] or default_color
     else:
         dcolor = color
     if dcolor != '':
@@ -93,10 +109,10 @@ def str_highlight_day(day, devents, hmethod, default_color, multiple, color, bol
 
 
 def str_week(week, today, collection=None,
-             hmethod=None, default_color=None, multiple=None, color=None,
-             highlight_event_days=False, locale=None, bold_for_light_color=True):
+             hmethod=None, default_color=None, multiple=None, multiple_on_overflow=False,
+             color=None, highlight_event_days=False, locale=None, bold_for_light_color=True):
     """returns a string representing one week,
-    if for day == today colour is reversed
+    if for day == today color is reversed
 
     :param week: list of 7 datetime.date objects (one week)
     :type day: list()
@@ -111,10 +127,11 @@ def str_week(week, today, collection=None,
         if day == today:
             day = style(str(day.day).rjust(2), reverse=True)
         elif highlight_event_days:
-            devents = list(collection.get_events_on(day, minimal=True))
+            devents = list(collection.get_calendars_on(day))
             if len(devents) > 0:
-                day = str_highlight_day(day, devents, hmethod, default_color,
-                                        multiple, color, bold_for_light_color)
+                day = str_highlight_day(day, devents, hmethod, default_color, multiple,
+                                        multiple_on_overflow, color, bold_for_light_color,
+                                        collection)
             else:
                 day = str(day.day).rjust(2)
         else:
@@ -123,16 +140,18 @@ def str_week(week, today, collection=None,
     return strweek
 
 
-def vertical_month(month=datetime.date.today().month,
-                   year=datetime.date.today().year,
-                   today=datetime.date.today(),
+def vertical_month(month=None,
+                   year=None,
+                   today=None,
                    weeknumber=False,
                    count=3,
                    firstweekday=0,
+                   monthdisplay='firstday',
                    collection=None,
                    hmethod='fg',
                    default_color='',
                    multiple='',
+                   multiple_on_overflow='',
                    color='',
                    highlight_event_days=False,
                    locale=None,
@@ -157,26 +176,37 @@ def vertical_month(month=datetime.date.today().month,
               ANSI (color) escape strings
     :rtype: list() of str()
     """
+    if month is None:
+        month = dt.date.today().month
+    if year is None:
+        year = dt.date.today().year
+    if today is None:
+        today = dt.date.today()
 
-    khal = list()
+    khal = []
     w_number = '  ' if weeknumber == 'right' else ''
     calendar.setfirstweekday(firstweekday)
     weekheaders = get_weekheader(firstweekday)
-    khal.append(style('    ' + weekheaders + ' ' + w_number, bold=True))
+    month_abbr_len = get_month_abbr_len()
+    khal.append(style(' ' * month_abbr_len + weekheaders + ' ' + w_number, bold=True))
     _calendar = calendar.Calendar(firstweekday)
     for _ in range(count):
         for week in _calendar.monthdatescalendar(year, month):
-            new_month = len([day for day in week if day.day == 1])
-            strweek = str_week(week, today, collection, hmethod, default_color,
-                               multiple, color, highlight_event_days, locale, bold_for_light_color)
-            if new_month:
-                m_name = style(calendar.month_abbr[week[6].month].ljust(4), bold=True)
-            elif weeknumber == 'left':
-                m_name = style(' {:2} '.format(getweeknumber(week[0])), bold=True)
+            if monthdisplay == 'firstday':
+                new_month = len([day for day in week if day.day == 1])
             else:
-                m_name = '    '
+                new_month = len(week if week[0].day <= 7 else [])
+            strweek = str_week(week, today, collection, hmethod, default_color,
+                               multiple, multiple_on_overflow, color, highlight_event_days, locale,
+                               bold_for_light_color)
+            if new_month:
+                m_name = style(calendar.month_abbr[week[6].month].ljust(month_abbr_len), bold=True)
+            elif weeknumber == 'left':
+                m_name = style(str(getweeknumber(week[0])).center(month_abbr_len), bold=True)
+            else:
+                m_name = ' ' * month_abbr_len
             if weeknumber == 'right':
-                w_number = style('{:2}'.format(getweeknumber(week[0])), bold=True)
+                w_number = style(f'{getweeknumber(week[0]):2}', bold=True)
             else:
                 w_number = ''
 
