@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -70,8 +70,8 @@ def multi_calendar_select(ctx, include_calendars, exclude_calendars):
         for cal_name in include_calendars:
             if cal_name not in ctx.obj['conf']['calendars']:
                 raise click.BadParameter(
-                    'Unknown calendar {}, run `khal printcalendars` to get a '
-                    'list of all configured calendars.'.format(cal_name)
+                    f'Unknown calendar {cal_name}, run `khal printcalendars` '
+                    'to get a list of all configured calendars.'
                 )
 
         selection.update(include_calendars)
@@ -107,8 +107,8 @@ def _select_one_calendar_callback(ctx, option, calendar):
 def _calendar_select_callback(ctx, option, calendar):
     if calendar and calendar not in ctx.obj['conf']['calendars']:
         raise click.BadParameter(
-            'Unknown calendar {}, run `khal printcalendars` to get a '
-            'list of all configured calendars.'.format(calendar)
+            f'Unknown calendar {calendar}, run `khal printcalendars` to get a '
+            'list of all configured calendars.'
         )
     return calendar
 
@@ -155,7 +155,7 @@ def global_options(f):
 def build_collection(conf, selection):
     """build and return a khalendar.CalendarCollection from the configuration"""
     try:
-        props = dict()
+        props = {}
         for name, cal in conf['calendars'].items():
             if selection is None or name in selection:
                 props[name] = {
@@ -174,6 +174,7 @@ def build_collection(conf, selection):
             hmethod=conf['highlight_days']['method'],
             default_color=conf['highlight_days']['default_color'],
             multiple=conf['highlight_days']['multiple'],
+            multiple_on_overflow=conf['highlight_days']['multiple_on_overflow'],
             highlight_event_days=conf['default']['highlight_event_days'],
         )
     except FatalError as error:
@@ -185,7 +186,7 @@ def build_collection(conf, selection):
     return collection
 
 
-class _NoConfig(object):
+class _NoConfig:
     def __getitem__(self, key):
         logger.fatal(
             'Cannot find a config file. If you have no configuration file '
@@ -215,16 +216,16 @@ def prepare_context(ctx, config):
 def stringify_conf(conf):
     # since we have only two levels of recursion, a recursive function isn't
     # really worth it
-    out = list()
+    out = []
     for key, value in conf.items():
-        out.append('[{}]'.format(key))
+        out.append(f'[{key}]')
         for subkey, subvalue in value.items():
             if isinstance(subvalue, dict):
-                out.append('  [[{}]]'.format(subkey))
+                out.append(f'  [[{subkey}]]')
                 for subsubkey, subsubvalue in subvalue.items():
-                    out.append('    {}: {}'.format(subsubkey, subsubvalue))
+                    out.append(f'    {subsubkey}: {subsubvalue}')
             else:
-                out.append('  {}: {}'.format(subkey, subvalue))
+                out.append(f'  {subkey}: {subvalue}')
     return '\n'.join(out)
 
 
@@ -279,6 +280,7 @@ def _get_cli():
                 hmethod=ctx.obj['conf']['highlight_days']['method'],
                 default_color=ctx.obj['conf']['highlight_days']['default_color'],
                 multiple=ctx.obj['conf']['highlight_days']['multiple'],
+                multiple_on_overflow=ctx.obj['conf']['highlight_days']['multiple_on_overflow'],
                 color=ctx.obj['conf']['highlight_days']['color'],
                 highlight_event_days=ctx.obj['conf']['default']['highlight_event_days'],
                 bold_for_light_color=ctx.obj['conf']['view']['bold_for_light_color'],
@@ -323,7 +325,11 @@ def _get_cli():
                 conf=ctx.obj['conf'],
                 env={"calendars": ctx.obj['conf']['calendars']}
             )
-            click.echo('\n'.join(event_column))
+            if event_column:
+                click.echo('\n'.join(event_column))
+            else:
+                logger.debug('No events found')
+
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -441,8 +447,15 @@ def _get_cli():
             if not ics:
                 ics_strs = (sys.stdin.read(),)
                 if not batch:
-                    if os.stat('/dev/tty').st_mode & stat.S_IFCHR > 0:
-                        sys.stdin = open('/dev/tty', 'r')
+
+                    def isatty(_file):
+                        try:
+                            return _file.isatty()
+                        except Exception:
+                            return False
+
+                    if isatty(sys.stdin) and os.stat('/dev/tty').st_mode & stat.S_IFCHR > 0:
+                        sys.stdin = open('/dev/tty')
                     else:
                         logger.warning('/dev/tty does not exist, importing might not work')
             else:
@@ -519,7 +532,7 @@ def _get_cli():
                     'longdatetimeformat', 'datetimeformat', 'longdateformat',
                     'dateformat', 'timeformat']:
                 dt_str = time.strftime(ctx.obj['conf']['locale'][strftime_format])
-                click.echo('{}: {}'.format(strftime_format, dt_str))
+                click.echo(f'{strftime_format}: {dt_str}')
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -568,7 +581,7 @@ def _get_cli():
                 multi_calendar_select(ctx, include_calendar, exclude_calendar)
             )
             events = sorted(collection.search(search_string))
-            event_column = list()
+            event_column = []
             term_width, _ = get_terminal_size()
             now = dt.datetime.now()
             env = {"calendars": ctx.obj['conf']['calendars']}
@@ -579,7 +592,10 @@ def _get_cli():
                              bold_for_light_color=ctx.obj['conf']['view']['bold_for_light_color'])
                      for d in desc]
                 )
-            click.echo('\n'.join(event_column))
+            if event_column:
+                click.echo('\n'.join(event_column))
+            else:
+                logger.debug('No events found')
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
@@ -640,7 +656,8 @@ def _get_cli():
                 conf=ctx.obj['conf'],
                 env={"calendars": ctx.obj['conf']['calendars']}
             )
-            click.echo('\n'.join(rows))
+            if rows:
+                click.echo('\n'.join(rows))
         except FatalError as error:
             logger.debug(error, exc_info=True)
             logger.fatal(error)
