@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2017 Christian Geier et al.
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,39 +20,44 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-import icalendar
-from click import confirm, echo, style, prompt
-
-from .khalendar.vdir import Item
-from .exceptions import ConfigurationError
+import datetime as dt
+import logging
+import os
+import re
+import textwrap
+from collections import OrderedDict, defaultdict
+from shutil import get_terminal_size
+from typing import Iterable, List, Tuple
 
 import pytz
+from click import confirm, echo, prompt, style
 
-from collections import defaultdict, OrderedDict
-from shutil import get_terminal_size
-
-from datetime import time, timedelta, datetime, date
-import os
-import textwrap
-
-from khal import utils, calendar_display
-from khal.khalendar.exceptions import ReadOnlyCalendarError, DuplicateUid
-from khal.exceptions import FatalError
+from khal import (__productname__, __version__, calendar_display,
+                  parse_datetime, utils)
+from khal.exceptions import DateTimeParseError, FatalError
+from khal.khalendar import CalendarCollection
 from khal.khalendar.event import Event
-from khal.khalendar.backend import sort_key
-from khal import __version__, __productname__
-from khal.log import logger
+from khal.khalendar.exceptions import DuplicateUid, ReadOnlyCalendarError
+
+from .exceptions import ConfigurationError
+from .icalendar import cal_from_ics
+from .icalendar import new_event as new_vevent
+from .icalendar import sort_key as sort_vevent_key
+from .icalendar import split_ics
+from .khalendar.vdir import Item
 from .terminal import merge_columns
 
+logger = logging.getLogger('khal')
 
-def format_day(day, format_string, locale, attributes=None):
+
+def format_day(day, format_string: str, locale, attributes=None):
     if attributes is None:
         attributes = {}
 
     attributes["date"] = day.strftime(locale['dateformat'])
     attributes["date-long"] = day.strftime(locale['longdateformat'])
 
-    attributes["name"] = utils.construct_daynames(day)
+    attributes["name"] = parse_datetime.construct_daynames(day)
 
     colors = {"reset": style("", reset=True), "bold": style("", bold=True, reset=False)}
     for c in ["black", "red", "green", "yellow", "blue", "magenta", "cyan", "white"]:
@@ -71,9 +76,11 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
              conf=None,
              firstweekday=0,
              weeknumber=False,
+             monthdisplay='firstday',
              hmethod='fg',
              default_color='',
              multiple='',
+             multiple_on_overflow=False,
              color='',
              highlight_event_days=0,
              full=False,
@@ -104,15 +111,19 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
         width=rwidth,
         env=env,
     )
+    if not event_column:
+        event_column = [style('No events', bold=True)]
     calendar_column = calendar_display.vertical_month(
         month=start.month,
         year=start.year,
         count=max(3, (end.year - start.year) * 12 + end.month - start.month + 1),
         firstweekday=firstweekday, weeknumber=weeknumber,
+        monthdisplay=monthdisplay,
         collection=collection,
         hmethod=hmethod,
         default_color=default_color,
         multiple=multiple,
+        multiple_on_overflow=multiple_on_overflow,
         color=color,
         highlight_event_days=highlight_event_days,
         locale=locale,
@@ -120,24 +131,25 @@ def calendar(collection, agenda_format=None, notstarted=False, once=False, dater
     return merge_columns(calendar_column, event_column, width=lwidth)
 
 
-def start_end_from_daterange(daterange, locale,
-                             default_timedelta_date=timedelta(days=1),
-                             default_timedelta_datetime=timedelta(hours=1)):
+def start_end_from_daterange(
+    daterange: Iterable[str],
+    locale: dict,
+    default_timedelta_date=dt.timedelta(days=1),
+    default_timedelta_datetime=dt.timedelta(hours=1),
+) -> Tuple[dt.datetime, dt.datetime]:
     """
     convert a string description of a daterange into start and end datetime
 
     if no description is given, return (today, today + default_timedelta_date)
 
     :param daterange: an iterable of strings that describes `daterange`
-    :type daterange: tuple
     :param locale: locale settings
-    :type locale: dict
     """
     if not daterange:
-        start = datetime(*date.today().timetuple()[:3])
+        start = dt.datetime(*dt.date.today().timetuple()[:3])
         end = start + default_timedelta_date
     else:
-        start, end, allday = utils.guessrangefstr(
+        start, end, allday = parse_datetime.guessrangefstr(
             daterange, locale, default_timedelta_date=default_timedelta_date,
             default_timedelta_datetime=default_timedelta_datetime,
         )
@@ -145,26 +157,29 @@ def start_end_from_daterange(daterange, locale,
 
 
 def get_events_between(
-        collection, locale, start, end, agenda_format=None, notstarted=False,
-        env=None, width=None, seen=None, original_start=None):
+    collection: CalendarCollection,
+    locale: dict,
+    start: dt.datetime,
+    end: dt.datetime,
+    agenda_format: str,
+    notstarted: bool,
+    env: dict,
+    width,
+    seen,
+    original_start: dt.datetime,
+) -> List[str]:
     """returns a list of events scheduled between start and end. Start and end
     are strings or datetimes (of some kind).
 
     :param collection:
-    :type collection: khalendar.CalendarCollection
     :param start: the start datetime
     :param end: the end datetime
     :param agenda_format: a format string that can be used in python string formatting
-    :type  agenda_format: str
     :param env: a collection of "static" values like calendar names and color
-    :type env: dict
     :param nostarted: True if each event should start after start (instead of
     be active between start and end)
-    :type nostarted: bool
     :param original_start: start datetime to compare against of notstarted is set
-    :type original_start: datetime.datetime
     :returns: a list to be printed as the agenda for the given days
-    :rtype: list(str)
     """
     assert not (notstarted and not original_start)
 
@@ -187,9 +202,9 @@ def get_events_between(
         # to understand what's going on here this way
         if notstarted:
             if event.allday and event.start < original_start.date():
-                    continue
+                continue
             elif not event.allday and event.start_local < original_start:
-                    continue
+                continue
         if seen is not None and event.uid in seen:
             continue
 
@@ -208,11 +223,22 @@ def get_events_between(
     return event_list
 
 
-def khal_list(collection, daterange=None, conf=None, agenda_format=None,
-              day_format=None, once=False, notstarted=False, width=False,
-              env=None, datepoint=None):
-    assert daterange is not None or datepoint is not None
+def khal_list(
+    collection,
+    daterange: Iterable[str] = None,
+    conf: dict = None,
+    agenda_format=None,
+    day_format: str=None,
+    once=False,
+    notstarted: bool = False,
+    width: bool = False,
+    env=None,
+    datepoint=None,
+):
     """returns a list of all events in `daterange`"""
+    assert daterange is not None or datepoint is not None
+    assert conf is not None
+
     # because empty strings are also Falsish
     if agenda_format is None:
         agenda_format = conf['view']['agenda_event_format']
@@ -225,27 +251,29 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
             default_timedelta_date=conf['default']['timedelta'],
             default_timedelta_datetime=conf['default']['timedelta'],
         )
-        logger.debug('Getting all events between {} and {}'.format(start, end))
+        logger.debug(f'Getting all events between {start} and {end}')
 
     elif datepoint is not None:
         if not datepoint:
             datepoint = ['now']
         try:
-            start, allday = utils.guessdatetimefstr(datepoint, conf['locale'], date.today())
+            start, allday = parse_datetime.guessdatetimefstr(
+                datepoint, conf['locale'], dt.date.today(),
+            )
         except ValueError:
-            raise FatalError('Invalid value of `{}` for a datetime'.format(' '.join(datepoint)))
+            raise FatalError('Invalid value of `{' '.join(datepoint)}` for a datetime')
         if allday:
-            logger.debug('Got date {}'.format(start))
+            logger.debug(f'Got date {start}')
             raise FatalError('Please supply a datetime, not a date.')
-        end = start + timedelta(seconds=1)
+        end = start + dt.timedelta(seconds=1)
         if day_format is None:
             day_format = style(
                 start.strftime(conf['locale']['longdatetimeformat']),
                 bold=True,
             )
-        logger.debug('Getting all events between {} and {}'.format(start, end))
+        logger.debug(f'Getting all events between {start} and {end}')
 
-    event_column = []
+    event_column: List[str] = []
     once = set() if once else None
     if env is None:
         env = {}
@@ -255,7 +283,7 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
         if start.date() == end.date():
             day_end = end
         else:
-            day_end = datetime.combine(start.date(), time.max)
+            day_end = dt.datetime.combine(start.date(), dt.time.max)
         current_events = get_events_between(
             collection, locale=conf['locale'], agenda_format=agenda_format, start=start,
             end=day_end, notstarted=notstarted, original_start=original_start,
@@ -264,22 +292,27 @@ def khal_list(collection, daterange=None, conf=None, agenda_format=None,
             width=width,
         )
         if day_format and (conf['default']['show_all_days'] or current_events):
+            if len(event_column) != 0 and conf['view']['blank_line_before_day']:
+                event_column.append('')
             event_column.append(format_day(start.date(), day_format, conf['locale']))
         event_column.extend(current_events)
-        start = datetime(*start.date().timetuple()[:3]) + timedelta(days=1)
+        start = dt.datetime(*start.date().timetuple()[:3]) + dt.timedelta(days=1)
 
-    if event_column == []:
-        event_column = [style('No events', bold=True)]
     return event_column
 
 
 def new_interactive(collection, calendar_name, conf, info, location=None,
                     categories=None, repeat=None, until=None, alarms=None,
-                    format=None, env=None):
+                    format=None, env=None, url=None):
     try:
-        info = utils.eventinfofstr(info, conf['locale'], adjust_reasonably=True, localize=False)
-    except ValueError:
-        info = dict()
+        info = parse_datetime.eventinfofstr(
+            info, conf['locale'],
+            conf['default']['default_event_duration'],
+            conf['default']['default_dayevent_duration'],
+            adjust_reasonably=True, localize=False,
+        )
+    except DateTimeParseError:
+        info = {}
 
     while True:
         summary = info.get('summary')
@@ -297,7 +330,7 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
             end_string = info["dtend"].strftime(conf['locale']['datetimeformat'])
             range_string = start_string + ' ' + end_string
         daterange = prompt("datetime range", default=range_string)
-        start, end, allday = utils.guessrangefstr(
+        start, end, allday = parse_datetime.guessrangefstr(
             daterange, conf['locale'], adjust_reasonably=True)
         info['dtstart'] = start
         info['dtend'] = end
@@ -323,7 +356,7 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
     event = new_from_args(
         collection, calendar_name, conf, format=format, env=env,
         location=location, categories=categories,
-        repeat=repeat, until=until, alarms=alarms,
+        repeat=repeat, until=until, alarms=alarms, url=url,
         **info)
 
     echo("event saved")
@@ -334,26 +367,33 @@ def new_interactive(collection, calendar_name, conf, info, location=None,
 
 def new_from_string(collection, calendar_name, conf, info, location=None,
                     categories=None, repeat=None, until=None, alarms=None,
-                    format=None, env=None):
+                    url=None, format=None, env=None):
     """construct a new event from a string and add it"""
-    info = utils.eventinfofstr(info, conf['locale'], adjust_reasonably=True, localize=False)
+    info = parse_datetime.eventinfofstr(
+        info, conf['locale'],
+        conf['default']['default_event_duration'],
+        conf['default']['default_dayevent_duration'],
+        adjust_reasonably=True, localize=False
+    )
     new_from_args(
         collection, calendar_name, conf, format=format, env=env,
         location=location, categories=categories, repeat=repeat,
-        until=until, alarms=alarms, **info
+        until=until, alarms=alarms, url=url, **info
     )
 
 
 def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
                   summary=None, description=None, allday=None, location=None,
                   categories=None, repeat=None, until=None, alarms=None,
-                  timezone=None, format=None, env=None):
+                  timezone=None, url=None, format=None, env=None):
     """Create a new event from arguments and add to vdirs"""
+    if isinstance(categories, str):
+        categories = [category.strip() for category in categories.split(',')]
     try:
-        event = utils.new_event(
+        event = new_vevent(
             locale=conf['locale'], location=location, categories=categories,
             repeat=repeat, until=until, alarms=alarms, dtstart=dtstart,
-            dtend=dtend, summary=summary, description=description, timezone=timezone,
+            dtend=dtend, summary=summary, description=description, timezone=timezone, url=url,
         )
     except ValueError as error:
         raise FatalError(error)
@@ -364,13 +404,13 @@ def new_from_args(collection, calendar_name, conf, dtstart=None, dtend=None,
         collection.new(event)
     except ReadOnlyCalendarError:
         raise FatalError(
-            'ERROR: Cannot modify calendar "{}" as it is read-only'.format(calendar_name)
+            f'ERROR: Cannot modify calendar "{calendar_name}" as it is read-only'
         )
 
     if conf['default']['print_new'] == 'event':
         if format is None:
             format = conf['view']['event_format']
-        echo(event.format(format, datetime.now(), env=env))
+        echo(event.format(format, dt.datetime.now(), env=env))
     elif conf['default']['print_new'] == 'path':
         path = os.path.join(
             collection._calendars[event.calendar]['path'],
@@ -411,8 +451,14 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
     options["categories"] = {"short": "c", "attr": "categories", "none": True}
     options["alarm"] = {"short": "a"}
     options["Delete"] = {"short": "D"}
+    options["url"] = {"short": "u", "attr": "url", "none": True}
+    # some output contains ansi escape sequences (probably only resets)
+    # if hitting enter, the output (including the escape sequence) gets parsed
+    # and fails the parsing. Therefore we remove ansi escape sequences before
+    # parsing.
+    ansi = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
 
-    now = datetime.now()
+    now = dt.datetime.now()
 
     while True:
         choice = present_options(options, prefix="Edit?", width=width)
@@ -434,10 +480,10 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
             current = event.format("{start} {end}", relative_to=now)
             value = prompt("datetime range", default=current)
             try:
-                start, end, allday = utils.guessrangefstr(value, locale)
+                start, end, allday = parse_datetime.guessrangefstr(ansi.sub('', value), locale)
                 event.update_start_end(start, end)
                 edited = True
-            except:
+            except:  # noqa
                 echo("error parsing range")
         elif choice == "repeat":
             recur = event.recurobject
@@ -452,13 +498,13 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
                 until = prompt('until (or "None")', until)
                 if until == 'None':
                     until = None
-                rrule = utils.rrulefstr(freq, until, locale)
+                rrule = parse_datetime.rrulefstr(freq, until, locale, event.start.tzinfo)
                 event.update_rrule(rrule)
             edited = True
         elif choice == "alarm":
             default_alarms = []
             for a in event.alarms:
-                s = utils.timedelta2str(-1 * a[0])
+                s = parse_datetime.timedelta2str(-1 * a[0])
                 default_alarms.append(s)
 
             default = ', '.join(default_alarms)
@@ -469,7 +515,7 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
                 alarm = ""
             alarm_list = []
             for a in alarm.split(","):
-                alarm_trig = -1 * utils.guesstimedeltafstr(a.strip())
+                alarm_trig = -1 * parse_datetime.guesstimedeltafstr(a.strip())
                 new_alarm = (alarm_trig, event.description)
                 alarm_list += [new_alarm]
             event.update_alarms(alarm_list)
@@ -489,7 +535,10 @@ def edit_event(event, collection, locale, allow_quit=False, width=80):
             value = prompt(question, default)
             if allow_none and value == "None":
                 value = ""
-            getattr(event, "update_" + attr)(value)
+            if attr == 'categories':
+                getattr(event, "update_" + attr)([cat.strip() for cat in value.split(',')])
+            else:
+                getattr(event, "update_" + attr)(value)
             edited = True
 
         if edited:
@@ -503,15 +552,15 @@ def edit(collection, search_string, locale, format=None, allow_past=False, conf=
             format = conf['view']['event_format']
 
     term_width, _ = get_terminal_size()
-    now = conf['locale']['local_timezone'].localize(datetime.now())
+    now = conf['locale']['local_timezone'].localize(dt.datetime.now())
 
     events = sorted(collection.search(search_string))
     for event in events:
         if not allow_past:
             if event.allday and event.end < now.date():
-                    continue
+                continue
             elif not event.allday and event.end_local < now:
-                    continue
+                continue
         event_text = textwrap.wrap(event.format(format, relative_to=now), term_width)
         echo(''.join(event_text))
         if not edit_event(event, collection, locale, allow_quit=True, width=term_width):
@@ -525,7 +574,7 @@ def interactive(collection, conf):
         collection, conf, title="select an event", description="do something")
     ui.start_pane(
         pane, pane.cleanup,
-        program_info='{0} v{1}'.format(__productname__, __version__),
+        program_info=f'{__productname__} v{__version__}',
         quit_keys=conf['keybindings']['quit'],
     )
 
@@ -543,7 +592,7 @@ def import_ics(collection, conf, ics, batch=False, random_uid=False, format=None
     """
     if format is None:
         format = conf['view']['event_format']
-    vevents = utils.split_ics(ics, random_uid, conf['locale']['default_timezone'])
+    vevents = split_ics(ics, random_uid, conf['locale']['default_timezone'])
     for vevent in vevents:
         import_event(vevent, collection, conf['locale'], batch, format, env)
 
@@ -557,11 +606,11 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
     """
     # print all sub-events
     if not batch:
-        for item in icalendar.Calendar.from_ical(vevent).walk():
+        for item in cal_from_ics(vevent).walk():
             if item.name == 'VEVENT':
                 event = Event.fromVEvents(
                     [item], calendar=collection.default_calendar_name, locale=locale)
-                echo(event.format(format, datetime.now(), env=env))
+                echo(event.format(format, dt.datetime.now(), env=env))
 
     # get the calendar to insert into
     if not collection.writable_names:
@@ -573,11 +622,11 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
     else:
         calendar_names = sorted(collection.writable_names)
         choices = ', '.join(
-            ['{}({})'.format(name, num) for num, name in enumerate(calendar_names)])
+            [f'{name}({num})' for num, name in enumerate(calendar_names)])
         while True:
             value = prompt(
                 "Which calendar do you want to import to? (unique prefixes are fine)\n"
-                "{}".format(choices),
+                f"{choices}",
                 default=collection.default_calendar_name,
             )
             try:
@@ -591,7 +640,7 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
             echo('invalid choice')
     assert calendar_name in collection.writable_names
 
-    if batch or confirm("Do you want to import this event into `{}`?".format(calendar_name)):
+    if batch or confirm(f"Do you want to import this event into `{calendar_name}`?"):
         try:
             collection.new(Item(vevent), collection=calendar_name)
         except DuplicateUid:
@@ -599,23 +648,23 @@ def import_event(vevent, collection, locale, batch, format=None, env=None):
                     "An event with the same UID already exists. Do you want to update it?"):
                 collection.force_update(Item(vevent), collection=calendar_name)
             else:
-                logger.warning("Not importing event with UID `{}`".format(event.uid))
+                logger.warning(f"Not importing event with UID `{event.uid}`")
 
 
 def print_ics(conf, name, ics, format):
     if format is None:
-        format = conf['view']['agenda_event_format']
-    cal = icalendar.Calendar.from_ical(ics)
+        format = conf['view']['event_format']
+    cal = cal_from_ics(ics)
     events = [item for item in cal.walk() if item.name == 'VEVENT']
     events_grouped = defaultdict(list)
     for event in events:
         events_grouped[event['UID']].append(event)
 
-    vevents = list()
+    vevents = []
     for uid in events_grouped:
-        vevents.append(sorted(events_grouped[uid], key=sort_key))
+        vevents.append(sorted(events_grouped[uid], key=sort_vevent_key))
 
-    echo('{} events found in {}'.format(len(vevents), name))
+    echo(f'{len(vevents)} events found in {name}')
     for sub_event in vevents:
         event = Event.fromVEvents(sub_event, locale=conf['locale'])
-        echo(event.format(format, datetime.now()))
+        echo(event.format(format, dt.datetime.now()))
