@@ -1,16 +1,15 @@
 import datetime as dt
 from textwrap import dedent
 
-from freezegun import freeze_time
 import pytest
+from freezegun import freeze_time
 
-from khal.khalendar.vdir import Item
-from khal.controllers import import_ics, khal_list, start_end_from_daterange
 from khal import exceptions
+from khal.controllers import import_ics, khal_list, start_end_from_daterange
+from khal.khalendar.vdir import Item
 
-from .utils import _get_text
 from . import utils
-
+from .utils import _get_text
 
 today = dt.date.today()
 yesterday = today - dt.timedelta(days=1)
@@ -89,8 +88,8 @@ class TestGetAgenda:
             'DTEND:20110908T170000\r\n'
             'END:VEVENT\r\n'
         ), utils.cal1))
-        assert 'no events' in '\n'.join(
-            khal_list(coll, [], conf, agenda_format=event_format, day_format="{name}")).lower()
+        assert '\n'.join(khal_list(coll, [], conf,
+                         agenda_format=event_format, day_format="{name}")).lower() == ''
 
 
 class TestImport:
