@@ -3,7 +3,8 @@ import datetime as dt
 import pytest
 import pytz
 from freezegun import freeze_time
-from icalendar import vRecur, vText
+from icalendar import Parameters, vCalAddress, vRecur, vText
+
 from khal.khalendar.event import (AllDayEvent, Event, FloatingEvent,
                                   LocalizedEvent, create_timezone)
 
@@ -15,6 +16,8 @@ EVENT_KWARGS = {'calendar': 'foobar', 'locale': LOCALE_BERLIN}
 LIST_FORMAT = '{calendar-color}{cancelled}{start-end-time-style} {title}{repeat-symbol}'
 SEARCH_FORMAT = '{calendar-color}{cancelled}{start-long}{to-style}' + \
     '{end-necessary-long} {title}{repeat-symbol}'
+FORMAT_CALENDAR = ('{calendar-color}{cancelled}{start-end-time-style} ({calendar}) '
+                   '{title} [{location}]{repeat-symbol}')
 
 
 def test_no_initialization():
@@ -46,6 +49,19 @@ def test_raw_dt():
     assert event.organizer == ''
 
 
+def test_calendar_in_format():
+    """test if the calendar is included in event.format() if specified in the FORMAT
+
+    see #1121
+    """
+    event_dt = _get_text('event_dt_simple')
+    start = BERLIN.localize(dt.datetime(2014, 4, 9, 9, 30))
+    end = BERLIN.localize(dt.datetime(2014, 4, 9, 10, 30))
+    event = Event.fromString(event_dt, start=start, end=end, **EVENT_KWARGS)
+    assert event.format(FORMAT_CALENDAR, dt.date(2014, 4, 9)) == \
+        '09:30-10:30 (foobar) An Event []\x1b[0m'
+
+
 def test_update_simple():
     event = Event.fromString(_get_text('event_dt_simple'), **EVENT_KWARGS)
     event_updated = Event.fromString(_get_text('event_dt_simple_updated'), **EVENT_KWARGS)
@@ -56,6 +72,17 @@ def test_update_simple():
     assert normalize_component(event.raw) == normalize_component(event_updated.raw)
 
 
+def test_add_url():
+    event = Event.fromString(_get_text('event_dt_simple'), **EVENT_KWARGS)
+    event.update_url('https://github.com/pimutils/khal')
+    assert 'URL:https://github.com/pimutils/khal' in event.raw
+
+
+def test_get_url():
+    event = Event.fromString(_get_text('event_dt_url'), **EVENT_KWARGS)
+    assert event.url == "https://github.com/pimutils/khal"
+
+
 def test_no_end():
     """reading an event with neither DTEND nor DURATION"""
     event = Event.fromString(_get_text('event_dt_no_end'), **EVENT_KWARGS)
@@ -77,6 +104,12 @@ def test_do_not_save_empty_description():
     assert 'DESCRIPTION' not in event.raw
 
 
+def test_do_not_save_empty_url():
+    event = Event.fromString(_get_text('event_dt_simple'), **EVENT_KWARGS)
+    event.update_url('')
+    assert 'URL' not in event.raw
+
+
 def test_remove_existing_location_if_set_to_empty():
     event = Event.fromString(_get_text('event_dt_simple_updated'), **EVENT_KWARGS)
     event.update_location('')
@@ -89,6 +122,12 @@ def test_remove_existing_description_if_set_to_empty():
     assert 'DESCRIPTION' not in event.raw
 
 
+def test_remove_existing_url_if_set_to_empty():
+    event = Event.fromString(_get_text('event_dt_url'), **EVENT_KWARGS)
+    event.update_url('')
+    assert 'URL' not in event.raw
+
+
 def test_update_remove_categories():
     event = Event.fromString(_get_text('event_dt_simple_updated'), **EVENT_KWARGS)
     event_nocat = Event.fromString(_get_text('event_dt_simple_nocat'), **EVENT_KWARGS)
@@ -456,6 +495,41 @@ def test_event_alarm():
     assert event.alarms == [(dt.timedelta(-1, 82800), vText('new event'))]
 
 
+def test_event_attendees():
+    event = Event.fromString(_get_text('event_dt_simple'), **EVENT_KWARGS)
+    assert event.attendees == ""
+    event.update_attendees(["this-does@not-exist.de", ])
+    assert event.attendees == "this-does@not-exist.de"
+    assert isinstance(event._vevents[event.ref].get('ATTENDEE', []), list)
+    assert isinstance(event._vevents[event.ref].get('ATTENDEE', [])[0], vCalAddress)
+    assert str(event._vevents[event.ref].get('ATTENDEE', [])[0]) == "MAILTO:this-does@not-exist.de"
+
+    event.update_attendees(["this-does@not-exist.de", "also-does@not-exist.de"])
+    assert event.attendees == "this-does@not-exist.de, also-does@not-exist.de"
+
+    assert isinstance(event._vevents[event.ref].get('ATTENDEE', []), list)
+    assert len(event._vevents[event.ref].get('ATTENDEE', [])) == 2
+    assert isinstance(event._vevents[event.ref].get('ATTENDEE', [])[0], vCalAddress)
+
+    # test if parameters from existing vCalAddress objects will be preserved
+    new_address = vCalAddress("MAILTO:mail.address@not-exist.de")
+    new_address.params = Parameters(
+        {'CN': 'Real Name',
+         'PARTSTAT': 'NEEDS-ACTION',
+         'ROLE': 'REQ-PARTICIPANT',
+         'RSVP': 'TRUE'}
+    )
+    event._vevents[event.ref]['ATTENDEE'] = [new_address, ]
+    event.update_attendees(["another.mailaddress@not-exist.de", "mail.address@not-exist.de"])
+    assert event.attendees == "mail.address@not-exist.de, another.mailaddress@not-exist.de"
+    address = [a for a in event._vevents[event.ref].get('ATTENDEE', [])
+               if str(a) == "MAILTO:mail.address@not-exist.de"]
+    assert len(address) == 1
+    address = address[0]
+    assert address.params.get('CN', None) is not None
+    assert address.params['CN'] == "Real Name"
+
+
 def test_create_timezone_static():
     gmt = pytz.timezone('Etc/GMT-8')
     assert create_timezone(gmt).to_ical().split() == [
@@ -499,3 +573,59 @@ UID:V042MJ8B3SJNFXQOJL6P53OFMHJE8Z3VZWOU
 END:VEVENT
 END:VCALENDAR"""
         )
+
+
+def test_sort_date_vs_datetime():
+    event1 = Event.fromString(_get_text('event_d'), **EVENT_KWARGS)
+    event2 = Event.fromString(_get_text('event_dt_floating'), **EVENT_KWARGS)
+    assert event1 < event2
+
+
+def test_sort_event_start():
+    event_dt = _get_text('event_dt_simple')
+    start = BERLIN.localize(dt.datetime(2014, 4, 9, 9, 45))
+    end = BERLIN.localize(dt.datetime(2014, 4, 9, 10, 30))
+    event1 = Event.fromString(event_dt, **EVENT_KWARGS)
+    event2 = Event.fromString(event_dt, start=start, end=end, **EVENT_KWARGS)
+    assert event1 < event2
+
+
+def test_sort_event_end():
+    event_dt = _get_text('event_dt_simple')
+    start = BERLIN.localize(dt.datetime(2014, 4, 9, 9, 30))
+    end = BERLIN.localize(dt.datetime(2014, 4, 9, 10, 45))
+    event1 = Event.fromString(event_dt, **EVENT_KWARGS)
+    event2 = Event.fromString(event_dt, start=start, end=end, **EVENT_KWARGS)
+    assert event1 < event2
+
+
+def test_sort_event_summary():
+    event_dt = _get_text('event_dt_simple')
+    event1 = Event.fromString(event_dt, **EVENT_KWARGS)
+    event2 = Event.fromString(event_dt, **EVENT_KWARGS)
+    event2.update_summary("ZZZ")
+    assert event1 < event2
+
+
+def test_create_timezone_in_future():
+    """Events too far into the future (after the next DST transition) used
+    to be created with invalid timezones"""
+    with freeze_time('2019-03-31'):
+        assert create_timezone(
+            pytz.timezone('Europe/Amsterdam'),
+            dt.datetime(2022, 1, 1, 18, 0)).to_ical().split() == [
+                b'BEGIN:VTIMEZONE',
+                b'TZID:Europe/Amsterdam',
+                b'BEGIN:STANDARD',
+                b'DTSTART;VALUE=DATE-TIME:20211031T020000',
+                b'TZNAME:CET',
+                b'TZOFFSETFROM:+0200',
+                b'TZOFFSETTO:+0100',
+                b'END:STANDARD',
+                b'BEGIN:DAYLIGHT',
+                b'DTSTART;VALUE=DATE-TIME:20220327T030000',
+                b'TZNAME:CEST',
+                b'TZOFFSETFROM:+0100',
+                b'TZOFFSETTO:+0200',
+                b'END:DAYLIGHT',
+                b'END:VTIMEZONE']
