@@ -1,4 +1,4 @@
-# Copyright (c) 2013-2021 khal contributors
+# Copyright (c) 2013-2022 khal contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,11 +23,14 @@ import os
 import time
 
 import pytest
+
 from khal.khalendar import vdir
 
 
-@pytest.mark.xfail
-def test_etag(tmpdir):
+def test_etag(tmpdir, sleep_time):
+    if sleep_time > 0.01:
+        pytest.xfail("This environment requires sleeping")
+
     fpath = os.path.join(str(tmpdir), 'foo')
 
     file_ = open(fpath, 'w')
@@ -42,22 +45,19 @@ def test_etag(tmpdir):
 
     new_etag = vdir.get_etag_from_file(fpath)
 
-    try:
-        assert old_etag != new_etag
-    except AssertionError:
-        pytest.xfail(
-            "Do we need to sleep?"
-        )
+    assert old_etag != new_etag
 
 
-def test_etag_sync(tmpdir):
+def test_etag_sync(tmpdir, sleep_time):
+    if sleep_time > 0.01:
+        pytest.xfail("This environment requires sleeping")
+
     fpath = os.path.join(str(tmpdir), 'foo')
 
     file_ = open(fpath, 'w')
     file_.write('foo')
     file_.close()
     os.sync()
-
     old_etag = vdir.get_etag_from_file(fpath)
 
     file_ = open(fpath, 'w')
@@ -86,3 +86,19 @@ def test_etag_sleep(tmpdir, sleep_time):
     new_etag = vdir.get_etag_from_file(fpath)
 
     assert old_etag != new_etag
+
+
+def test_get_href_from_uid():
+    # Test UID with unsafe characters
+    uid = "V042MJ8B3SJNFXQOJL6P53OFMHJE8Z3VZWÈÉ@pimutils.org"
+    first_href = vdir._generate_href(uid)
+    second_href = vdir._generate_href(uid)
+    assert first_href == second_href
+
+    # test UID with safe characters
+    uid = "V042MJ8B3SJNFXQOJL6P53OFMHJE8Z3VZWOU@pimutils.org"
+    href = vdir._generate_href(uid)
+    assert href == uid
+
+    href = vdir._generate_href()
+    assert href is not None
